/******************************************************************
 * PalmOS includes.
 ******************************************************************/
#include <Common.h>
#include <System/SysAll.h>
#include <System/SysEvtMgr.h>
#include <UI/UIAll.h>

/******************************************************************
 * Application includes.
 ******************************************************************/
#include "callback.h"
#include "resource.h"

#include "Timesheet.h"
#include "Globals.h"

#include "Category.h"
#include "Database.h"

#include "DetailForm.h"
#include "EditCatForm.h"
#include "EditCatEntryForm.h"

/******************************************************************
 * Event handler for the Edit Category form.
 ******************************************************************/
Boolean
EditCatEntryFormEventHandler(EventPtr event)
{
  Boolean handled = false;
  Int result;
  FormPtr formPtr;

  if (event->eType == ctlSelectEvent)
    {
      /* a control button was pressed and released */
      if (event->data.ctlEnter.controlID == buttonID_tsOk)
        {
          /* check if we're create a new category or renaming an existing category */
          if (TSEditCatEntryIdx == 0)
            {
              /* add the new entry to the category */
              if ((result = EditCatEntryAddEntry()) == -1)
                {
                  /* do nothing, user hasn't entered valid new category name */
                  handled = true;
                  return handled;
                }               /* else fall through to code below */
              /* show the 1-down category edit form again */
              FrmReturnToForm(formID_tsEditCat);
              /* and re-initialise it now that there's a new entry */
              EditCatFormInit(result - 1, true);
              FrmDrawForm(FrmGetActiveForm());
              /* also need to update appropriate list on the 2-down Detail form */
              if (TSEditCatRecIdx == 1)
                {
                  /* refill the list and update the current selection */
                  DetailFormPopupListAdd(listID_tsClient, listpopupID_tsClient, 1,
                                       TSAppPrefs.numCatEntries[0], result);
                }
              else if (TSEditCatRecIdx == 2)
                {
                  /* refill the list and update the current selection */
                  DetailFormPopupListAdd(listID_tsProject, listpopupID_tsProject, 2,
                                       TSAppPrefs.numCatEntries[1], result);
                }
              else if (TSEditCatRecIdx == 3)
                {
                  /* refill the list and update the current selection */
                  DetailFormPopupListAdd(listID_tsTask, listpopupID_tsTask, 3,
                                       TSAppPrefs.numCatEntries[2], result);
                }
            }
          else
            {
              /* rename an existing, selected category */
              if ((result = EditCatEntryRenameEntry()) == -1)
                {
                  /* do nothing, user hasn't entered valid category name */
                  handled = true;
                  return handled;
                }
              else if (result == 0)
                {
                  /* user hasn't changed category details, do nothing but return to previous form */
                  FrmReturnToForm(formID_tsEditCat);
                  handled = true;
                  return handled;
                }
              /* show the 1-down category edit form again */
              FrmReturnToForm(formID_tsEditCat);
              /* and re-initialise it now that there's a new entry */
              EditCatFormInit(result - 1, true);
              FrmDrawForm(FrmGetActiveForm());
              /* also need to update appropriate list on the 2-down Detail form */
              if (TSEditCatRecIdx == 1)
                {
                  /* refill the list and update the current selection */
                  DetailFormPopupListRename(listID_tsClient, listpopupID_tsClient, 1,
                    TSAppPrefs.numCatEntries[0], TSEditCatEntryIdx, result);
                }
              else if (TSEditCatRecIdx == 2)
                {
                  /* refill the list and update the current selection */
                  DetailFormPopupListRename(listID_tsProject, listpopupID_tsProject, 2,
                    TSAppPrefs.numCatEntries[1], TSEditCatEntryIdx, result);
                }
              else if (TSEditCatRecIdx == 3)
                {
                  /* refill the list and update the current selection */
                  DetailFormPopupListRename(listID_tsTask, listpopupID_tsTask, 3,
                    TSAppPrefs.numCatEntries[2], TSEditCatEntryIdx, result);
                }
            }
          handled = true;
        }
      else if (event->data.ctlEnter.controlID == buttonID_tsCancel)
        {
          /* cancel category form and show the detail form again */
          FrmReturnToForm(formID_tsEditCat);
          handled = true;
        }
    }
  else if (event->eType == frmOpenEvent)
    {
      /* the form has been opened */
      formPtr = FrmGetActiveForm();
      FrmDrawForm(formPtr);
      FrmSetFocus(formPtr, FrmGetObjectIndex(formPtr, fieldID_tsCat));
      handled = true;
    }

  return handled;
}

/******************************************************************
 * Attempt to add a new user-entered entry to the category currently
 * being edited. Returns the entry index of the new entry if added,
 * otherwise returns -1 if the entry wasn't added for some reason.
 ******************************************************************/
Int
EditCatEntryAddEntry(void)
{
  CharPtr newCat;
  Byte idx;
  Int newCatIdx;
  Byte freeTransIdx;
  FormPtr formPtr;
  FieldPtr fieldPtr;
  VoidHand catHand;
  TSCatRecType *catPtr;

  formPtr = FrmGetActiveForm();
  fieldPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, fieldID_tsCat));
  /* check the user has actually entered a new category */
  newCat = FldGetTextPtr(fieldPtr);
  if (newCat == NULL || newCat[0] == NULL || IsAllSpace(newCat))
    {
      /* string appears to be empty, alert the user and return */
      FrmAlert(alertID_tsEmptyCat);
      return -1;
    }
  /* attempt to insert the new entry into the entry array alphabetically */
  if ((newCatIdx = CatInsertEntry(TSEditCatRecIdx, TSAppPrefs.numCatEntries[TSEditCatRecIdx - 1], newCat)) < 0)
    {
      /* user attempts to add category that already exists */
      FrmAlert(alertID_tsExistingCat);
      return -1;
    }
  /* now comes the hard part... find a free entry in the category trans table for this new category */
  catHand = DmGetRecord(TSDatabase, TSEditCatRecIdx);
  catPtr = MemHandleLock(catHand);
  do
    {
      for (idx = 0, freeTransIdx = TSMaxCatEntries; idx < TSMaxCatEntries; idx += 1)
        {
          if (catPtr->transTable[idx] == TSMaxCatEntries)
            {
              freeTransIdx = idx;
              break;
            }
        }
      /* do we have a free index, or is a database reorg required? */
      if (freeTransIdx == TSMaxCatEntries)
        {
          /* reorg is required */
          if (!ReorgDatabase(catPtr, TSEditCatRecIdx))
            {
              /* reorg has freed no entries, category really is completely full */
              FrmAlert(alertID_tsCatFull);
              MemHandleUnlock(catHand);
              DmReleaseRecord(TSDatabase, TSEditCatRecIdx, false);
              return -1;
            }
        }
    }
  while (freeTransIdx == TSMaxCatEntries);
  /* have created a new entry, now map the entry into the trans table */
  idx = newCatIdx;              /* type conversion */
  DmWrite(catPtr, (VoidPtr) (&(catPtr->transTable[freeTransIdx])) - (VoidPtr) (catPtr), &idx, sizeof(Byte));
  /* successfully added new category */
  TSAppPrefs.numCatEntries[TSEditCatRecIdx - 1] += 1;
  MemHandleUnlock(catHand);
  DmReleaseRecord(TSDatabase, TSEditCatRecIdx, true);
  return newCatIdx;
}

/******************************************************************
 * Delete a category name from the category being edited, handles
 * remapping the deleted category to 'none'.
 ******************************************************************/
void
EditCatEntryDeleteEntry(UInt deleteCatIdx)
{
  Byte idx;
  Byte zero;
  FormPtr formPtr;
  VoidHand catHand;
  TSCatRecType *catPtr;

  formPtr = FrmGetActiveForm();
  /* delete the specified category, adjusting category entries to retain alphabetic order */
  catHand = CatDeleteEntry(TSEditCatRecIdx, TSAppPrefs.numCatEntries[TSEditCatRecIdx - 1], deleteCatIdx);
  catPtr = MemHandleLock(catHand);
  /* remap all deleted transTable entries to the 0, 'none' entry */
  zero = 0;
  for (idx = 1; idx < TSMaxCatEntries; idx += 1)
    {
      if (catPtr->transTable[idx] == 255)       /* marks the entries that have just been deleted */
        {
          /* remap entry to 0 'none' entry */
          DmWrite(catPtr, sizeof(Byte) * idx, &zero, sizeof(Byte));
        }
    }
  /* successfully deleted category */
  TSAppPrefs.numCatEntries[TSEditCatRecIdx - 1] -= 1;
  MemHandleUnlock(catHand);
  DmReleaseRecord(TSDatabase, TSEditCatRecIdx, true);
}

/******************************************************************
 * Modify an existing category entry. Returns -1 if the new category
 * name the user has entered is invalid, or 0 if the new category
 * name is the *same* as the existing name (the user has changed
 * nothing). If the category has been renamed successfully the index
 * of the
 ******************************************************************/
Int
EditCatEntryRenameEntry(void)
{
  CharPtr renamedCat;
  Int insertCatIdx;
  Byte idx;
  Byte catIdx;
  FormPtr formPtr;
  FieldPtr fieldPtr;
  VoidHand catHand;
  TSCatRecType *catPtr;

  formPtr = FrmGetActiveForm();
  fieldPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, fieldID_tsCat));
  /* check the user has actually entered a category name */
  renamedCat = FldGetTextPtr(fieldPtr);
  if (renamedCat == NULL || renamedCat[0] == NULL || IsAllSpace(renamedCat))
    {
      /* string appears to be empty, alert the user and return */
      FrmAlert(alertID_tsEmptyCat);
      return -1;
    }
  /* check the category name is actually different */
  catHand = DmGetRecord(TSDatabase, TSEditCatRecIdx);
  catPtr = MemHandleLock(catHand);
  if (StrCompare(renamedCat, catPtr->cats[TSEditCatEntryIdx]) == 0)
    {
      /* don't have to do anything, user has changed nothing ;) */
      MemHandleUnlock(catHand);
      DmReleaseRecord(TSDatabase, TSEditCatRecIdx, false);
      return 0;
    }
  /* check the category name isn't a special category name */
  if (StrCompare(renamedCat, catPtr->cats[0]) == 0 ||
      StrCompare(renamedCat, catPtr->cats[TSAppPrefs.numCatEntries[TSEditCatRecIdx - 1] - 1]) == 0)
    {
      FrmAlert(alertID_tsReservedCat);
      MemHandleUnlock(catHand);
      DmReleaseRecord(TSDatabase, TSEditCatRecIdx, false);
      return -1;
    }
  /* delete the renamed entry (also modifies trans table) */
  MemHandleUnlock(catHand);
  DmReleaseRecord(TSDatabase, TSEditCatRecIdx, true);
  CatDeleteEntry(TSEditCatRecIdx, TSAppPrefs.numCatEntries[TSEditCatRecIdx - 1], TSEditCatEntryIdx);
  /* insert the renamed entry into the entry array alphabetically */
  insertCatIdx = CatInsertEntry(TSEditCatRecIdx,
                                TSAppPrefs.numCatEntries[TSEditCatRecIdx - 1] - 1 /* just deleted entry */ ,
                                renamedCat);
  catHand = DmGetRecord(TSDatabase, TSEditCatRecIdx);
  catPtr = MemHandleLock(catHand);
  if (insertCatIdx < 0)
    {
      /* category has been renamed to match an existing category which already exists */
      catIdx = -insertCatIdx - 1;       /* type conversion */
      /* map all deleted transTable entries to category that matches rename */
      for (idx = 1; idx < TSMaxCatEntries; idx += 1)
        {
          if (catPtr->transTable[idx] == 255)
            {
              DmWrite(catPtr, sizeof(Byte) * idx, &catIdx, sizeof(Byte));
            }
        }
      /* now have one less category entry */
      TSAppPrefs.numCatEntries[TSEditCatRecIdx - 1] -= 1;
    }
  else
    {
      /* category has been renamed to another unique category name, have inserted name, update trans table */
      catIdx = insertCatIdx;    /* type conversion */
      /* map all deleted transTable entries to category that matches new category name */
      for (idx = 1; idx < TSMaxCatEntries; idx += 1)
        {
          if (catPtr->transTable[idx] == 255)
            {
              DmWrite(catPtr, sizeof(Byte) * idx, &catIdx, sizeof(Byte));
            }
        }
    }
  /* successfully renamed category */
  MemHandleUnlock(catHand);
  DmReleaseRecord(TSDatabase, TSEditCatRecIdx, true);
  return catIdx;
}

/******************************************************************
 * Returns true if the passed string contains whitespace only.
 ******************************************************************/
Boolean
IsAllSpace(CharPtr s)
{
  while ((*s) != NULL)
    {
      if (!ISSPACE(*s))
        {
          return false;
        }
      s += 1;
    }
  return true;
}

/*** End of file ***/