/******************************************************************
 * PalmOS includes.
 ******************************************************************/
#include <Common.h>
#include <System/SysAll.h>
#include <System/SysEvtMgr.h>
#include <UI/UIAll.h>

/******************************************************************
 * Application includes.
 ******************************************************************/
#include "resource.h"
#include "Timesheet.h"
#include "Globals.h"

#include "MainForm.h"
#include "PrefForm.h"
#include "UIUtil.h"

/******************************************************************
 *
 ******************************************************************/
Boolean
PrefFormEventHandler(EventPtr event)
{
  FormPtr formPtr;
  Boolean handled = false;

  if (event->eType == ctlSelectEvent)
    {
      if (event->data.ctlEnter.controlID == buttonID_tsOk)
        {
          /* Save the potentially altered prefs */
          PrefFormSavePrefs(FrmGetActiveForm());
          /* Return to the previous form */
          FrmReturnToForm(formID_tsMain);
          /* Refresh the main display appearance as prefs may have changed */
          FormUpdateDay(&TSPresentDayDate);
          MainFormRefresh();
          handled = true;
        }
      else if (event->data.ctlEnter.controlID == buttonID_tsCancel)
        {
          /* return to the previous form */
          FrmReturnToForm(0);
          handled = true;
        }
    }
  else if (event->eType == frmOpenEvent)
    {
      /* the Pref form has been opened */
      formPtr = FrmGetActiveForm();
      PrefFormInit(formPtr);
      FrmDrawForm(formPtr);
      handled = true;
    }

  return handled;
}

/******************************************************************
 * Set up the prefs form depending on the current app prefs
 ******************************************************************/
void
PrefFormInit(FormPtr formPtr)
{
  ControlPtr controlPtr;

  /* set the appropriate group button for date format */
  FrmSetControlGroupSelection(formPtr, groupID_tsFormatGroup,
      buttonID_tsLongFormat + (TSAppPrefs.prefFlags & TSPrefShortDateFlag));
  /* set the underline chargeable clients pref */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsUnderline));
  /* note below is correct, set if user does NOT want client underlining */
  CtlSetValue(controlPtr, TSAppPrefs.prefFlags & TSPrefUlineClientFlag);
  /* set the chargeable totals pref */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsChargeTotals));
  /* note below is correct, set if user does NOT want chargeable totals */
  CtlSetValue(controlPtr, !(TSAppPrefs.prefFlags & TSPrefChargeTotalsFlag));
  /* set the auto categories pref */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsAutoCats));
  CtlSetValue(controlPtr, (TSAppPrefs.prefFlags & TSPrefAutoCatsFlag));
  /* set the auto durations pref */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsAutoDur));
  CtlSetValue(controlPtr, (TSAppPrefs.prefFlags & TSPrefAutoDurFlag));
  /* set the default new entry chargeable pref */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsCharge));
  CtlSetValue(controlPtr, (TSAppPrefs.prefFlags & TSPrefDefaultChargeFlag));
  /* Set the keep timing pref */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsKeepTiming));
  CtlSetValue(controlPtr, (TSAppPrefs.prefFlags & TSPrefAlwaysTimeFlag));
}

/******************************************************************
 * Save the update prefs back to the global app preferences
 ******************************************************************/
void
PrefFormSavePrefs(FormPtr formPtr)
{
  ControlPtr controlPtr;

  /* save the date format selection */
  if (FrmGetObjectId(formPtr, FrmGetControlGroupSelection(formPtr, groupID_tsFormatGroup)) == buttonID_tsShortFormat)
    {
      /* set flag */
      TSAppPrefs.prefFlags |= TSPrefShortDateFlag;
    }
  else
    {
      /* clear flag */
      TSAppPrefs.prefFlags &= (~TSPrefShortDateFlag);
    }
  /* save the underline chargeable clients pref */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsUnderline));
  if (CtlGetValue(controlPtr))
    {
      /* set flag */
      TSAppPrefs.prefFlags |= TSPrefUlineClientFlag;
    }
  else
    {
      /* clear flag */
      TSAppPrefs.prefFlags &= (~TSPrefUlineClientFlag);
    }
  /* save the chargeable totals pref */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsChargeTotals));
  if (!CtlGetValue(controlPtr))
    {
      /* set flag */
      TSAppPrefs.prefFlags |= TSPrefChargeTotalsFlag;
    }
  else
    {
      /* clear flag */
      TSAppPrefs.prefFlags &= (~TSPrefChargeTotalsFlag);
    }    
  /* save the auto categories pref */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsAutoCats));
  if (CtlGetValue(controlPtr))
    {
      /* set flag */
      TSAppPrefs.prefFlags |= TSPrefAutoCatsFlag;
    }
  else
    {
      /* clear flag */
      TSAppPrefs.prefFlags &= (~TSPrefAutoCatsFlag);
    }
  /* save the auto durations pref */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsAutoDur));
  if (CtlGetValue(controlPtr))
    {
      /* set flag */
      TSAppPrefs.prefFlags |= TSPrefAutoDurFlag;
    }
  else
    {
      /* clear flag */
      TSAppPrefs.prefFlags &= (~TSPrefAutoDurFlag);
    }
  /* save the default new entry chargeable pref */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsCharge));
  if (CtlGetValue(controlPtr))
    {
      /* set flag */
      TSAppPrefs.prefFlags |= TSPrefDefaultChargeFlag;
    }
  else
    {
      /* clear flag */
      TSAppPrefs.prefFlags &= (~TSPrefDefaultChargeFlag);
    }
  /* save the default keep timing flag */
  controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, checkboxID_tsKeepTiming));
  if (CtlGetValue(controlPtr))
    {
      /* set flag */
      TSAppPrefs.prefFlags |= TSPrefAlwaysTimeFlag;
    }
  else
    {
      /* clear flag */
      TSAppPrefs.prefFlags &= (~TSPrefAlwaysTimeFlag);
    }
}

/*** End of file ***/